import Bio
import sys
import math
from Bio import SeqIO

#declare input
background_files = []
foreground_files = []
b_kmer_len=0

#initialize input
if len(sys.argv) > 4: 
	b_kmer_len=int(sys.argv[1])
	for ind in range(3,3+int(sys.argv[2])):
		background_files.append(sys.argv[ind])
	for ind in range(3+int(sys.argv[2]),len(sys.argv)):
		foreground_files.append(sys.argv[ind])
else:
	print("Insufficient input\n")


#define nucleic acid dictionary
nucl_dict = {"A" : 0, "C" : 1, "G" : 2, "T" : 3 }
inv_nucl_dict = {0 : "A", 1 : "C", 2 : "G", 3 : "T" }


#save original output channel
original_stdout = sys.stdout

#create array of background counts
background_frequency = [0.0 for i in range(0,pow(4,b_kmer_len))]



#define function which calculates the background frequency of a given read S
def construct_frequency(S):
	ret = 0.0
	
	for kmer_start in range(0, len(S)-b_kmer_len+1):
		tmp_pos = 0
		
		for kmer_rel_pos in range(0,b_kmer_len):
			tmp_pos += nucl_dict[ S[kmer_start+kmer_rel_pos] ] * pow(4,kmer_rel_pos)
		
		ret += background_frequency[tmp_pos]
	
	return ret



#construct background frequency array
normalization = 0
for data_file in background_files:
	for read in SeqIO.parse(data_file+".fastq","fastq"):
		s = read.seq
		
		for kmer_start in range(0,len(s)-b_kmer_len+1):
			tmp_cnt = 0
			is_N = 0
			
			for kmer_rel_pos in range(0,b_kmer_len):
				if(s[kmer_start+kmer_rel_pos] != 'N'):
					tmp_cnt += nucl_dict[ s[kmer_start + kmer_rel_pos] ] * pow(4,kmer_rel_pos)
				else:
					is_N = 1
					
			#only use sequence if there is no N in it
			if( is_N == 0 ):
				background_frequency[tmp_cnt] += 1.0
				normalization += 1
			
		
	

#normalize and print log of background likelihood of kmer
with open("background_frequencies_"+str(b_kmer_len)+"mer.dat","w") as f:
	sys.stdout = f
	for i in range(0,pow(4,b_kmer_len)):
		background_frequency[i] = background_frequency[i]/normalization
		s = ""
		tmpI = i
		for j in range(0,b_kmer_len):
			tmpN = tmpI % 4
			s = s + inv_nucl_dict[tmpN]
			tmpI -= tmpN
			tmpI /= 4
		print(s,background_frequency[i])
	sys.stdout = original_stdout



#create foreground sequence dictionaries, print in data format for RBAPBindnSeq
background_normalization = 0
number_kmers = 0
for data_file in foreground_files:
	foreground_dict = {}
	
	#create foreground dictionary
	for seq_record in SeqIO.parse(data_file+".fastq","fastq"):
		if seq_record.seq in foreground_dict:
			foreground_dict[seq_record.seq] += 1
		else:
			foreground_dict[seq_record.seq] = 1
		
	
	#print read files with background frequencies
	with open(data_file+"_"+str(b_kmer_len)+"mer.dat","w") as f:
		sys.stdout = f
		for s in foreground_dict:
			if('N' not in s):
				tmp_bgr = construct_frequency(s)
				if tmp_bgr > 0.0 and len(s) >= b_kmer_len:
					background_normalization += foreground_dict[s]*math.log(tmp_bgr)
					number_kmers += foreground_dict[s]*(len(s) - b_kmer_len + 1)
					print(len(s),foreground_dict[s],tmp_bgr,s)
		sys.stdout = original_stdout
		
	
print("background normalization for ", b_kmer_len, "mer: ", background_normalization, "\nNumber of possible ", b_kmer_len, "mers: ", number_kmers)

